@@ -23,6 +23,9 @@
 
 #include "missing.h"
 
+#define DEFAULT_TEXT_DOMAIN	"sudo"
+#include "gettext.h"
+
 #if defined(HAVE_DECL_SYS_SIGLIST) && HAVE_DECL_SYS_SIGLIST == 1
 # define my_sys_siglist	sys_siglist
 #elif defined(HAVE_DECL__SYS_SIGLIST) && HAVE_DECL__SYS_SIGLIST == 1
@@ -41,5 +44,5 @@ strsignal(int signo)
 {
     if (signo > 0 && signo < NSIG)
 	return (char *)my_sys_siglist[signo];
-    return "Unknown signal";
+    return _("Unknown signal");
 }
