@@ -108,58 +108,56 @@
 #define I_VERIFYPW              53
 #define def_noexec              (sudo_defs_table[54].sd_un.flag)
 #define I_NOEXEC                54
-#define def_noexec_file         (sudo_defs_table[55].sd_un.str)
-#define I_NOEXEC_FILE           55
-#define def_ignore_local_sudoers (sudo_defs_table[56].sd_un.flag)
-#define I_IGNORE_LOCAL_SUDOERS  56
-#define def_closefrom           (sudo_defs_table[57].sd_un.ival)
-#define I_CLOSEFROM             57
-#define def_closefrom_override  (sudo_defs_table[58].sd_un.flag)
-#define I_CLOSEFROM_OVERRIDE    58
-#define def_setenv              (sudo_defs_table[59].sd_un.flag)
-#define I_SETENV                59
-#define def_env_reset           (sudo_defs_table[60].sd_un.flag)
-#define I_ENV_RESET             60
-#define def_env_check           (sudo_defs_table[61].sd_un.list)
-#define I_ENV_CHECK             61
-#define def_env_delete          (sudo_defs_table[62].sd_un.list)
-#define I_ENV_DELETE            62
-#define def_env_keep            (sudo_defs_table[63].sd_un.list)
-#define I_ENV_KEEP              63
-#define def_role                (sudo_defs_table[64].sd_un.str)
-#define I_ROLE                  64
-#define def_type                (sudo_defs_table[65].sd_un.str)
-#define I_TYPE                  65
-#define def_env_file            (sudo_defs_table[66].sd_un.str)
-#define I_ENV_FILE              66
-#define def_sudoers_locale      (sudo_defs_table[67].sd_un.str)
-#define I_SUDOERS_LOCALE        67
-#define def_visiblepw           (sudo_defs_table[68].sd_un.flag)
-#define I_VISIBLEPW             68
-#define def_pwfeedback          (sudo_defs_table[69].sd_un.flag)
-#define I_PWFEEDBACK            69
-#define def_fast_glob           (sudo_defs_table[70].sd_un.flag)
-#define I_FAST_GLOB             70
-#define def_umask_override      (sudo_defs_table[71].sd_un.flag)
-#define I_UMASK_OVERRIDE        71
-#define def_log_input           (sudo_defs_table[72].sd_un.flag)
-#define I_LOG_INPUT             72
-#define def_log_output          (sudo_defs_table[73].sd_un.flag)
-#define I_LOG_OUTPUT            73
-#define def_compress_io         (sudo_defs_table[74].sd_un.flag)
-#define I_COMPRESS_IO           74
-#define def_use_pty             (sudo_defs_table[75].sd_un.flag)
-#define I_USE_PTY               75
-#define def_group_plugin        (sudo_defs_table[76].sd_un.str)
-#define I_GROUP_PLUGIN          76
-#define def_iolog_dir           (sudo_defs_table[77].sd_un.str)
-#define I_IOLOG_DIR             77
-#define def_iolog_file          (sudo_defs_table[78].sd_un.str)
-#define I_IOLOG_FILE            78
-#define def_set_utmp            (sudo_defs_table[79].sd_un.flag)
-#define I_SET_UTMP              79
-#define def_utmp_runas          (sudo_defs_table[80].sd_un.flag)
-#define I_UTMP_RUNAS            80
+#define def_ignore_local_sudoers (sudo_defs_table[55].sd_un.flag)
+#define I_IGNORE_LOCAL_SUDOERS  55
+#define def_closefrom           (sudo_defs_table[56].sd_un.ival)
+#define I_CLOSEFROM             56
+#define def_closefrom_override  (sudo_defs_table[57].sd_un.flag)
+#define I_CLOSEFROM_OVERRIDE    57
+#define def_setenv              (sudo_defs_table[58].sd_un.flag)
+#define I_SETENV                58
+#define def_env_reset           (sudo_defs_table[59].sd_un.flag)
+#define I_ENV_RESET             59
+#define def_env_check           (sudo_defs_table[60].sd_un.list)
+#define I_ENV_CHECK             60
+#define def_env_delete          (sudo_defs_table[61].sd_un.list)
+#define I_ENV_DELETE            61
+#define def_env_keep            (sudo_defs_table[62].sd_un.list)
+#define I_ENV_KEEP              62
+#define def_role                (sudo_defs_table[63].sd_un.str)
+#define I_ROLE                  63
+#define def_type                (sudo_defs_table[64].sd_un.str)
+#define I_TYPE                  64
+#define def_env_file            (sudo_defs_table[65].sd_un.str)
+#define I_ENV_FILE              65
+#define def_sudoers_locale      (sudo_defs_table[66].sd_un.str)
+#define I_SUDOERS_LOCALE        66
+#define def_visiblepw           (sudo_defs_table[67].sd_un.flag)
+#define I_VISIBLEPW             67
+#define def_pwfeedback          (sudo_defs_table[68].sd_un.flag)
+#define I_PWFEEDBACK            68
+#define def_fast_glob           (sudo_defs_table[69].sd_un.flag)
+#define I_FAST_GLOB             69
+#define def_umask_override      (sudo_defs_table[70].sd_un.flag)
+#define I_UMASK_OVERRIDE        70
+#define def_log_input           (sudo_defs_table[71].sd_un.flag)
+#define I_LOG_INPUT             71
+#define def_log_output          (sudo_defs_table[72].sd_un.flag)
+#define I_LOG_OUTPUT            72
+#define def_compress_io         (sudo_defs_table[73].sd_un.flag)
+#define I_COMPRESS_IO           73
+#define def_use_pty             (sudo_defs_table[74].sd_un.flag)
+#define I_USE_PTY               74
+#define def_group_plugin        (sudo_defs_table[75].sd_un.str)
+#define I_GROUP_PLUGIN          75
+#define def_iolog_dir           (sudo_defs_table[76].sd_un.str)
+#define I_IOLOG_DIR             76
+#define def_iolog_file          (sudo_defs_table[77].sd_un.str)
+#define I_IOLOG_FILE            77
+#define def_set_utmp            (sudo_defs_table[78].sd_un.flag)
+#define I_SET_UTMP              78
+#define def_utmp_runas          (sudo_defs_table[79].sd_un.flag)
+#define I_UTMP_RUNAS            79
 
 enum def_tuple {
 	never,
