@@ -35,12 +35,12 @@
 static void _warning(int, const char *, va_list);
        void plugin_cleanup(int);
 
-sigjmp_buf error_jmp;
+extern sigjmp_buf error_jmp;
 
 extern sudo_conv_t sudo_conv;
 
 void
-error(int eval, const char *fmt, ...)
+error2(int eval, const char *fmt, ...)
 {
     va_list ap;
 
@@ -52,7 +52,7 @@ error(int eval, const char *fmt, ...)
 }
 
 void
-errorx(int eval, const char *fmt, ...)
+errorx2(int eval, const char *fmt, ...)
 {
     va_list ap;
 
@@ -64,7 +64,7 @@ errorx(int eval, const char *fmt, ...)
 }
 
 void
-warning(const char *fmt, ...)
+warning2(const char *fmt, ...)
 {
     va_list ap;
 
@@ -74,7 +74,7 @@ warning(const char *fmt, ...)
 }
 
 void
-warningx(const char *fmt, ...)
+warningx2(const char *fmt, ...)
 {
     va_list ap;
     va_start(ap, fmt);
