@@ -43,6 +43,8 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#define SUDO_ERROR_WRAP 0
+
 #include "sudoers.h"
 #include "parse.h"
 #include "interfaces.h"
@@ -53,6 +55,8 @@ static int check_addr_printf(int msg_type, const char *fmt, ...);
 struct interface *interfaces;
 sudo_printf_t sudo_printf = check_addr_printf;
 
+sudo_conv_t sudo_conv;		/* NULL in non-plugin */
+
 static int
 check_addr(char *input)
 {
