@@ -26,6 +26,11 @@
 #  include <stdlib.h>
 # endif
 #endif /* STDC_HEADERS */
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#else
+# include "compat/stdbool.h"
+#endif /* HAVE_STDBOOL_H */
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif /* HAVE_STRING_H */
@@ -35,15 +40,20 @@
 #include <grp.h>
 #include <pwd.h>
 
+#define SUDO_ERROR_WRAP 0
+
 #include "list.h"
 #include "parse.h"
 #include "toke.h"
-#include "gram.h"
+#include "sudo_plugin.h"
+#include <gram.h>
 
 /*
  * TODO: test realloc
  */
 
+sudo_conv_t sudo_conv;		/* NULL in non-plugin */
+
 YYSTYPE yylval;
 
 struct fill_test {
