@@ -27,5 +27,5 @@ struct sudo_preload_table {
 } sudo_preload_table[] = {
     { "sudoers_policy", (void *) &sudoers_policy},
     { "sudoers_io", (void *) &sudoers_io},
-    { NULL, NULL }
+    { (const char *)0, (void *)0 }
 };
